@@ -1,7 +1,7 @@
 /*
  *  display.c - code for this nifty display.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ static int		display_result_line = 0;
 
 static char	*number_mod_labels[5] = {" DEC ", " HEX ", " OCT ", " BIN ", NULL}, 
 		*angle_mod_labels[4] = {" DEG ", " RAD ", " GRAD ", NULL},
-		*notation_mod_labels[3] = {" ALG ", " RPN ", NULL};	
+		*notation_mod_labels[4] = {" ALG ", " RPN ", " FORM ", NULL};	
 
 /*
  * display.c mainly consists of two parts: first display setup code
@@ -309,7 +309,7 @@ void display_module_base_create (char **module_label, char *mark_name, int activ
 	int		label_counter=0, counter;
 	GtkTextIter 	start, end;
 	GtkTextMark	*this_mark;
-	
+
 	if ((this_mark = gtk_text_buffer_get_mark (buffer, mark_name)) == NULL) return;
 	while (module_label[label_counter] != NULL) label_counter++;
 	for (counter = (label_counter-1); counter >= 0; counter--) {
@@ -528,7 +528,6 @@ void display_module_base_delete (char *mark_name, char **text)
 
 void display_change_option (int new_status, int opt_group)
 {
-	int	old_status;
 	double	display_value=0;
 	double 	*stack;
 	
@@ -538,7 +537,6 @@ void display_change_option (int new_status, int opt_group)
 			if (current_status.number == new_status) return;
 			display_value = display_result_get_double ();
 			stack = display_stack_get_yzt_double ();
-			old_status = current_status.number;
 			current_status.number = new_status;
 			display_result_set_double (display_value);
 			display_stack_set_yzt_double (stack);
@@ -550,7 +548,6 @@ void display_change_option (int new_status, int opt_group)
 			break;
 		case DISPLAY_OPT_ANGLE:
 			if (current_status.angle == new_status) return;
-			old_status = current_status.angle;
 			current_status.angle = new_status;
 			if ((prefs.vis_angle) && (prefs.mode == SCIENTIFIC_MODE)){
 				display_module_base_delete (DISPLAY_MARK_ANGLE, angle_mod_labels);
@@ -560,7 +557,6 @@ void display_change_option (int new_status, int opt_group)
 		case DISPLAY_OPT_NOTATION:
 			update_active_buttons (current_status.number, new_status);
 			if (current_status.notation == new_status) return;
-			old_status = current_status.notation;
 			current_status.notation = new_status;
 			if ((prefs.vis_notation) && (prefs.mode == SCIENTIFIC_MODE)){
 				display_module_base_delete (DISPLAY_MARK_NOTATION, notation_mod_labels);
@@ -639,8 +635,6 @@ void display_result_add_digit (char digit)
 	char			digit_as_string[2];
 	GtkTextIter		end;
 	
-	display_result_changed();
-	
 	digit_as_string[0] = digit;
 	digit_as_string[1] = '\0';
 	
@@ -758,7 +752,7 @@ double *display_stack_get_yzt_double ()
 	double_stack = (double *) malloc (display_result_line * sizeof(double));
 	string_stack = display_stack_get_yzt();
 	for (counter = 0; counter < display_result_line; counter++)
-		double_stack[counter] = string2double (string_stack[counter]);
+		double_stack[counter] = string2double (string_stack[counter], current_status.number);
 	return double_stack;
 }
 
@@ -786,43 +780,16 @@ void display_result_set_double (double value)
 {	
 	current_status.allow_arith_op = TRUE;
 	display_module_arith_label_update (' ');
-	
-	display_result_changed();
-	
 	display_set_line_double (value, display_result_line, "result");
 }
 
-/*
- * display_result_set_radiant - this function is used e.g. by trigonometric functions.
- *	pays attention to rad/deg/grad
- */
-
-void display_result_set_radiant (double value)
-{
-	switch (current_status.angle) {
-	case CS_DEG:
-		display_result_set_double (rad2deg(value)); 
-		break;
-	case CS_RAD:
-		display_result_set_double (value); 
-		break;
-	case CS_GRAD:
-		display_result_set_double (rad2grad(value));
-		break;
-	default:
-		fprintf (stderr, _("[%s] unknown angle base in function \"display_result_set_angle\". %s\n"), PROG_NAME, BUG_REPORT);
-	}
-}
-
 void display_result_set (char *string_value)
 {
 	GtkTextIter 		end;
 	
 	current_status.allow_arith_op = TRUE;
 	display_module_arith_label_update (' ');
-	
-	display_result_changed();
-	
+
 	/* at first clear the result field */
 	
 	display_delete_line (buffer, display_result_line, &end);
@@ -847,8 +814,10 @@ void display_result_feed (char *string)
 	int	counter;
 	
 	for (counter = 0; counter < strlen(string); counter++) {
-		if (is_valid_number(current_status.number, string[counter])) \
-			display_result_add_digit (string[counter]);
+		if (is_valid_number(current_status.number, string[counter])) 
+			/* g_ascii_toupper to convert lower hex chars to upper */
+			display_result_add_digit (
+				g_ascii_toupper(string[counter]));
 	}
 	if (string[0] == '-') display_result_toggle_sign (NULL);
 }
@@ -871,51 +840,26 @@ double display_result_get_double ()
 	double	ret_val;
 	
 	result_string = display_result_get();
-	ret_val = string2double (result_string);
+	ret_val = string2double (result_string, current_status.number);
 	g_free (result_string);
 		
 	return ret_val;
 }	
 
-
-/*
- * display_result_get_angle - this function is used e.g. by trigonometric functions.
- *	pays attention to rad/deg/grad
- */
-
-double display_result_get_rad_angle ()
-{
-	double		value;
-	
-	value = display_result_get_double();
-	switch (current_status.angle)
-	{
-	case CS_DEG:
-		return deg2rad(value); 
-	case CS_RAD:
-		return value;
-	case CS_GRAD:
-		return grad2rad (value);
-	default:
-		fprintf (stderr, _("[%s] unknown angle base in function \"display_result_get_rad_angle\". %s\n"), PROG_NAME, BUG_REPORT);
-	}
-	return -1;
-}
-
 void display_append_e (GtkToggleButton *button)
 {
 	GtkTextIter		end;
-	
-	display_result_changed();
-	
+
+	/* we have kind of a shortcut. we don't set to 0e+ but 1e+ */
 	if (current_status.number != CS_DEC) return;
-	if (current_status.calc_entry_start_new == FALSE) {
-		if (strstr (display_result_get(), "e+") == NULL) {
+	if ((current_status.calc_entry_start_new == FALSE) && 
+		(strcmp (display_result_get(), "0") != 0)) {
+		if (strstr (display_result_get(), "e") == NULL) {
 			display_get_line_end_iter (buffer, display_result_line, &end);
 			gtk_text_buffer_insert_with_tags_by_name (buffer, &end, "e+", -1, "result", NULL);
 		}
 	} else {
-		display_result_set ("0e+");
+		display_result_set ("1e+");
 		current_status.calc_entry_start_new = FALSE;
 	}
 	display_result_counter = get_display_number_length(current_status.number) - DISPLAY_RESULT_E_LENGTH;
@@ -925,9 +869,7 @@ void display_result_toggle_sign (GtkToggleButton *button)
 {
 	GtkTextIter		start, end;
 	char			*result_field, *e_pointer;
-	
-	display_result_changed();
-	
+
 	if (current_status.number != CS_DEC) return;
 	/* we could call display_result_get but we need start iterator later on anyway */
 	display_get_line_iters (buffer, display_result_line, &start, &end);
@@ -959,9 +901,7 @@ void display_result_toggle_sign (GtkToggleButton *button)
 void display_result_backspace ()
 {															
 	char	*current_entry;
-	
-	display_result_changed();
-	
+
 	if (current_status.calc_entry_start_new == TRUE) {
 		current_status.calc_entry_start_new = FALSE;
 		display_result_set ("0");
