@@ -1,7 +1,7 @@
 /*
  *  galculator.h - general definitions.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@ enum {
 enum {
 	CS_PAN,			/* _P_seudo _A_lgebraic _N_otation */
 	CS_RPN,			/* reverse polish notation */
-	/*CS_FORMULA,*/
+	CS_FORMULA,		/* formula entry */
 	NR_NOTATION_MODES
 };
 
@@ -104,12 +104,19 @@ enum {
 };
 
 enum {
-	NAME_COLUMN,
-	VALUE_COLUMN,
-	DESC_COLUMN,
+	CONST_NAME_COLUMN,
+	CONST_VALUE_COLUMN,
+	CONST_DESC_COLUMN,
 	NR_CONST_COLUMNS
 };
 
+enum {
+	UFUNC_NAME_COLUMN,
+	UFUNC_VARIABLE_COLUMN,
+	UFUNC_EXPRESSION_COLUMN,
+	NR_UFUNC_COLUMNS
+};
+
 enum {
 	DISPCTRL_NONE,
 	DISPCTRL_RIGHT,
@@ -121,31 +128,32 @@ enum {
 typedef struct {
 	unsigned	number:2;
 	unsigned	angle:2;
-	unsigned	notation:1;
+	unsigned	notation:2;
 	unsigned	fmod:2;
 	gboolean	calc_entry_start_new;
-	gboolean	rpn_have_result;
+	gboolean	rpn_stack_lift_enabled;
 	gboolean	allow_arith_op;
 } s_current_status;
 
 typedef struct {
 	char		*button_name;
+	/* for simplicity we put the display_names not in an array */
+	char		*display_names[4];
 	double		(*func[4])(double);
-	gboolean	is_trigonometric;
 } s_function_map;
 
 typedef struct {
 	char		*button_name;
-	double		constant;
-} s_constant_map;
-
-typedef struct {
-	char		*button_name;
+	char 		*display_string;
 	void		(*func)();
 } s_gfunc_map;
 
 typedef struct {
 	char		*button_name;
+	/* display_string: what to display in history or formula entry. hasn't 
+	 * to be the button label, e.g. "n!" and "!" 
+	 */
+	char 		*display_string;
 	int		operation;
 } s_operation_map;
 
@@ -160,6 +168,12 @@ typedef struct {
 	char		*value;
 } s_constant;
 
+typedef struct {
+	char 		*name;
+	char 		*variable;
+	char 		*expression;
+} s_user_function;
+
 typedef struct {
 	double		*data;
 	int		len;
@@ -169,6 +183,9 @@ extern s_array		memory;
 #include "config_file.h"
 extern s_preferences	prefs;
 extern s_constant 	*constant;
+extern s_user_function	*user_function;
 extern s_current_status	current_status;
+#include "calc_basic.h"
+extern ALG_OBJECT	*main_alg;
 
 #endif /* galculator.h */
