@@ -1,7 +1,7 @@
 /*
  *  general_functions.c - this and that.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "config_file.h"
 #include "callbacks.h"
 #include "ui.h"
+#include "flex_parser.h"
 
 #include <gtk/gtk.h>
 #include <glade/glade.h>
@@ -60,6 +61,17 @@ void error_message (char *message)
 void clear ()
 {
 	display_result_set (CLEARED_DISPLAY);
+	if (current_status.notation == CS_FORMULA) ui_formula_entry_set ("");
+}
+
+/* backspace. if formula_entry is active, backspace works anyway. but if we
+ * press GUI's backspace button this gives the expected result.
+ */
+
+void backspace ()
+{
+	if (current_status.notation == CS_FORMULA) ui_formula_entry_backspace();
+	else display_result_backspace();
 }
 
 /* clear all: display ("0"), calc_tree ... */
@@ -67,16 +79,27 @@ void clear ()
 void all_clear ()
 {
 	clear();
-	if (current_status.notation == CS_PAN) {
-		alg_free();
-		alg_init(0);
-	}
-	else {
-		rpn_free();
-		rpn_init(prefs.stack_size, 0);
-		display_stack_remove();
-		display_stack_create();
-		current_status.rpn_have_result = FALSE;
+	switch (current_status.notation) {
+		case CS_PAN:
+			alg_free(main_alg);
+			main_alg = alg_init(0);
+			break;
+		case CS_RPN:
+			rpn_free();
+			rpn_init(prefs.stack_size, 0);
+			display_stack_remove();
+			display_stack_create();
+			/* it's a stack lifting disabling functon */
+			current_status.rpn_stack_lift_enabled = FALSE;
+			break;
+		case CS_FORMULA:
+			alg_free(main_alg);
+			rpn_free();
+			display_stack_remove();
+			break;
+		default:
+			fprintf (stderr, _("[%s] unknown notation mode in function \"all_clear\". %s\n"), PROG_NAME, BUG_REPORT);
+			
 	}
 	display_module_bracket_label_update (RESET);
 }
@@ -290,18 +313,17 @@ void apply_preferences (s_preferences prefs)
 
 	display_update_tags ();
 	display_set_bkg_color (prefs.bkg_color);
-
+	
 	set_widget_visibility (main_window_xml, "menubar", prefs.show_menu);
 	menu_item = glade_xml_get_widget (main_window_xml, "show_menubar1");
 	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menu_item, prefs.show_menu);
 
-	
 	if (prefs.mode == BASIC_MODE) menu_item = 
 		glade_xml_get_widget (main_window_xml, "basic_mode");
 	else if (prefs.mode == SCIENTIFIC_MODE) menu_item = 
 		glade_xml_get_widget (main_window_xml, "scientific_mode");
 	gtk_menu_item_activate ((GtkMenuItem *) menu_item);
-	
+
 	set_all_buttons_size (prefs.button_width, prefs.button_height);
 
 	if (prefs.custom_button_font == TRUE) button_font = g_strdup (prefs.button_font);
@@ -341,12 +363,13 @@ void activate_menu_item (char *item_name)
 	
 	menu_item = (GtkMenuItem *) glade_xml_get_widget (main_window_xml, \
 		g_strstrip (g_ascii_strdown (item_name, -1)));
+	if (menu_item)
 	/* as we use this only for menu boxes, a simple activate is enough.
 	 * the extra magic in src/callbacks.c::on_scientific_mode_activate
 	 * is necessary only for checkboxmenuitems.
 	 */
-	gtk_menu_item_activate ((GtkMenuItem *) menu_item);
-
+		gtk_menu_item_activate ((GtkMenuItem *) menu_item);
+	else fprintf (stderr, _("[%s] failed to find widget %s in function \"activate_menu_item\". %s\n"), PROG_NAME, item_name, BUG_REPORT);
 }
 
 /* get_number_string - converts value to a string, with repect to base. returned
@@ -417,12 +440,13 @@ void gfunc_f1 (GtkToggleButton *button)
 	if (current_status.notation == CS_PAN) 
 		on_operation_button_clicked (button, NULL);
 	else {
-		current_status.rpn_have_result = FALSE;
 		display_result_set_double (rpn_stack_swapxy(
 			display_result_get_double()));
 		stack = rpn_stack_get (RPN_FINITE_STACK);
 		display_stack_set_yzt_double (stack);
 		free (stack);
+		current_status.rpn_stack_lift_enabled = TRUE;
+		current_status.calc_entry_start_new = TRUE;
 	}
 }
 
@@ -436,35 +460,35 @@ void gfunc_f2 (GtkToggleButton *button)
 	if (current_status.notation == CS_PAN)
 		on_operation_button_clicked (button, NULL);
 	else {
-		current_status.rpn_have_result = FALSE;
 		display_result_set_double (rpn_stack_rolldown(
 			display_result_get_double()));
 				stack = rpn_stack_get (RPN_FINITE_STACK);
 		display_stack_set_yzt_double (stack);
 		free (stack);
+		current_status.rpn_stack_lift_enabled = TRUE;
+		current_status.calc_entry_start_new = TRUE;
 	}
 }
 
-/*
- *
+/* rpn_stack_lift. The terminology used here originates from HP 15 Owner's
+ * handbook. If a button doesn't terminate digit entry, it "starts" one.
+ * Therefore those buttons have to push result on the stack, if stack lift is
+ * enabled (current_status.rpn_stack_lift_enabled).
+ * Buttons not terminating digit entry are:
+ *	0-9, A-F, ., EE, const, MR
  */
 
-void display_result_changed ()
+void rpn_stack_lift ()
 {
 	double	*stack;
 	
-	/* put a ev. result onto the stack */
-	/* an alternative idea would be to do this immediately after getting a result
-	   in calc_basic's rpn routines or in on_function_button_clicked. but if we
-	   use on_function_button_clicked twice, there are two results on the stack
-	   where we don't expect the older one to be there. therefore doing it this way
-	*/
-	if ((current_status.notation == CS_RPN) && (current_status.rpn_have_result == TRUE)) {
+	if ((current_status.notation == CS_RPN) && 
+		(current_status.rpn_stack_lift_enabled == TRUE)) {
 		rpn_stack_push (display_result_get_double ());
 		stack = rpn_stack_get (RPN_FINITE_STACK);
 		display_stack_set_yzt_double (stack);
 		free (stack);
-		current_status.rpn_have_result = FALSE;
+		current_status.rpn_stack_lift_enabled = FALSE;
 	}
 }
 
@@ -481,23 +505,31 @@ void remember_display_values()
 			stack[2] = prefs.rem_valuet;
 			display_stack_set_yzt (stack);
 			/* for the stack we have to update calc_basic */
-			rpn_stack_push (string2double(stack[2]));
-			rpn_stack_push (string2double(stack[1]));
-			rpn_stack_push (string2double(stack[0]));
+			rpn_stack_push (string2double(stack[2], current_status.number));
+			rpn_stack_push (string2double(stack[1], current_status.number));
+			rpn_stack_push (string2double(stack[0], current_status.number));
 		}
 	}
 }
 
 /*
  * string2double - this function makes a string to double conversion with 
- * 	respect to internal calculator settings. it uses axtof.
+ * 	respect to supplied number base. if number base < 0 we try to 
+ *	determine from string which should have then a 0? prefix.
+ *	it uses axtof.
  */
 
-double string2double (char *string)
+double string2double (char *string, int number_base)
 {
-	switch (current_status.number) {
+	char 	*end_ptr;
+	double	ret_val;
+	
+	switch (number_base) {
 		case CS_DEC:
-			return atof(string);
+			ret_val = strtod(string, &end_ptr);
+			if (*end_ptr != '\0')
+				fprintf (stderr, _("[%s] failed to convert %s to a number properly in function \"string2double\". Have you changed your locales? %s\n"), PACKAGE, string, BUG_REPORT);
+			return ret_val;
 			break;
 		case CS_HEX:
 			return axtof(string, 16, prefs.hex_bits, 
@@ -512,20 +544,84 @@ double string2double (char *string)
 				prefs.bin_signed);
 			break;
 		default:
-			fprintf (stderr, _("[%s] unknown number base in function \"display_result_get_double\". %s\n"), PROG_NAME, BUG_REPORT);
+			fprintf (stderr, _("[%s] unknown number base in function \"string2double\". %s\n"), PROG_NAME, BUG_REPORT);
 	}
 	return 0;
 }
 
-void update_dispctrl()
+
+void set_button_label_and_tooltip (GladeXML *xml, char *button_name, 
+	char *label, char *tooltip)
 {
-	/* just put one here and hide it afterwards. we need the button
-			for working key accelerators. */
-	if (prefs.mode == BASIC_MODE) 
-		ui_main_window_set_dispctrl (DISPCTRL_BOTTOM);
-	else if (current_status.notation == CS_RPN)
-		ui_main_window_set_dispctrl (DISPCTRL_RIGHTV);
-	else ui_main_window_set_dispctrl (DISPCTRL_RIGHT);
-	set_widget_visibility (dispctrl_xml, "table_dispctrl", 
-		prefs.vis_dispctrl);
+	GtkWidget	*w;
+	GtkTooltipsData	*tooltip_data;
+	
+	w = glade_xml_get_widget (xml, button_name);
+	if (w) {
+		gtk_button_set_label ((GtkButton *)w, label);
+		tooltip_data = gtk_tooltips_data_get (w);
+		g_free (tooltip_data->tip_text);
+		tooltip_data->tip_text = g_strdup (tooltip);
+	}
+}
+
+gboolean formula_entry_is_active (GtkWidget *window_widget)
+{
+	GtkWidget	*active_widget, *main_window;
+	
+	main_window = glade_xml_get_widget (main_window_xml, "main_window");
+	active_widget = gtk_window_get_focus ((GtkWindow *)main_window);
+	return (!strcmp (gtk_widget_get_name (gtk_widget_get_toplevel(window_widget)),"main_window") & 
+		!strcmp (gtk_widget_get_name (active_widget), "formula_entry"));
+}
+
+s_flex_parser_result compute_user_function (char *expression, char *variable, char *value)
+{
+	int			nr_constants;
+	s_flex_parser_result	result;
+	
+	nr_constants = 0;
+	while (constant[nr_constants].name != NULL) nr_constants++;
+	constant = (s_constant *) realloc (constant, (nr_constants + 2) * sizeof(s_constant));
+	constant[nr_constants + 1].name = NULL;
+	
+	constant[nr_constants].name = variable;
+	constant[nr_constants].value = value;
+	constant[nr_constants].desc = NULL;
+	
+	result = flex_parser(expression);
+	
+	constant = (s_constant *) realloc (constant, (nr_constants + 1) * sizeof(s_constant));
+	constant[nr_constants].name = NULL;
+	return result;
+}
+
+double x2rad (double x)
+{
+	switch (current_status.angle){
+	case CS_DEG:
+		return deg2rad(x); 
+	case CS_RAD:
+		return x;
+	case CS_GRAD:
+		return grad2rad (x);
+	default:
+		fprintf (stderr, _("[%s] unknown angle base in function \"x2rad\". %s\n"), PROG_NAME, BUG_REPORT);
+	}
+	return x;
+}	
+
+double rad2x (double rad)
+{
+	switch (current_status.angle) {
+	case CS_DEG:
+		return rad2deg(rad);
+	case CS_RAD:
+		return rad;
+	case CS_GRAD:
+		return rad2grad(rad);
+	default:
+		fprintf (stderr, _("[%s] unknown angle base in function \"rad2x\". %s\n"), PROG_NAME, BUG_REPORT);
+	}
+	return rad;
 }
