@@ -1,7 +1,7 @@
 /*
  *  ui.c - general user interface code.
  *	part of galculator
- *  	(c) 2002-2003 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,9 +37,12 @@
 GladeXML	*main_window_xml, *dispctrl_xml, *button_box_xml, *prefs_xml, 
 		*about_dialog_xml;
 char		dec_point[2];
-GtkListStore	*store;
+GtkListStore	*prefs_constant_store, *prefs_user_function_store;
 
 static void set_disp_ctrl_object_data ();
+static void set_all_dispctrl_buttons_property (GFunc func, gpointer data);
+static void set_all_normal_buttons_property (GFunc func, gpointer data);
+static void set_table_child_callback (gpointer data, gpointer user_data);
 
 /* active_buttons. bit mask, in which modes the corresponding button is active.
  * assume TRUE for all other bases/modes!
@@ -60,10 +63,10 @@ s_active_buttons active_buttons[] = {\
 	{"button_d", ~(AB_DEC | AB_BIN | AB_OCT)}, \
 	{"button_e", ~(AB_DEC | AB_BIN | AB_OCT)}, \
 	{"button_f", ~(AB_DEC | AB_BIN | AB_OCT)}, \
-	{"button_ee", AB_DEC | AB_RPN | AB_PAN}, \
-	{"button_sin", AB_DEC | AB_RPN | AB_PAN}, \
-	{"button_cos", AB_DEC | AB_RPN | AB_PAN}, \
-	{"button_tan", AB_DEC | AB_RPN | AB_PAN}, \
+	{"button_ee", AB_DEC}, \
+	{"button_sin", AB_DEC}, \
+	{"button_cos", AB_DEC}, \
+	{"button_tan", AB_DEC}, \
 	{"button_point", ~(AB_BIN | AB_OCT | AB_HEX)}, \
 	{"button_sign", ~(AB_BIN | AB_OCT | AB_HEX)}, \
 	{NULL}\
@@ -102,20 +105,25 @@ static void apply_object_data (s_operation_map operation_map[],
 {
 	int 		counter;
 	gpointer	*func;
+	GObject		*object;
 	
 	counter = 0;
 	while (operation_map[counter].button_name != NULL) {
-		g_object_set_data (G_OBJECT (glade_xml_get_widget (button_box_xml, 
-			operation_map[counter].button_name)),
-			"operation", GINT_TO_POINTER(operation_map[counter].operation));
+		object = G_OBJECT (glade_xml_get_widget (button_box_xml, 
+			operation_map[counter].button_name));
+		g_object_set_data (object, "operation",
+			GINT_TO_POINTER(operation_map[counter].operation));
+		g_object_set_data (object, "display_string",
+			operation_map[counter].display_string);
 		counter++;
 	}
 	
 	counter = 0;
 	while (gfunc_map[counter].button_name != NULL) {
-		g_object_set_data (G_OBJECT (glade_xml_get_widget (button_box_xml, 
-			gfunc_map[counter].button_name)),
-			"func", gfunc_map[counter].func);
+		object = G_OBJECT (glade_xml_get_widget (button_box_xml, 
+			gfunc_map[counter].button_name));
+ 		g_object_set_data (object, "display_string", gfunc_map[counter].display_string);
+		g_object_set_data (object, "func", gfunc_map[counter].func);
 		counter++;
 	};
 	
@@ -123,12 +131,10 @@ static void apply_object_data (s_operation_map operation_map[],
 	while (function_map[counter].button_name != NULL) {
 		func = (void *) malloc (sizeof (function_map[counter].func));
 		memcpy (func, function_map[counter].func, sizeof (function_map[counter].func));
-		g_object_set_data (G_OBJECT (glade_xml_get_widget (button_box_xml, 
-			function_map[counter].button_name)),
-			"func", func);
-		g_object_set_data (G_OBJECT (glade_xml_get_widget (button_box_xml, 
-			function_map[counter].button_name)),
-			"is_trigonometric", GINT_TO_POINTER((int)function_map[counter].is_trigonometric));		
+		object = G_OBJECT (glade_xml_get_widget (button_box_xml, 
+			function_map[counter].button_name));
+		g_object_set_data (object, "display_names", function_map[counter].display_names);
+		g_object_set_data (object, "func", func);	
 		counter++;
 	};
 }
@@ -140,41 +146,43 @@ static void apply_object_data (s_operation_map operation_map[],
 static void set_scientific_object_data ()
 {
 	s_operation_map	operation_map[] = {
-		{"button_pow", '^'},
-		{"button_lsh", '<'},
-		{"button_mod", 'm'},
-		{"button_and", '&'},
-		{"button_or", '|'},
-		{"button_xor", 'x'},
-		{"button_enter", '='},
-		{"button_plus", '+'},
-		{"button_minus", '-'},
-		{"button_mult", '*'},
-		{"button_div", '/'},
-		{"button_percent", '%'},
-		{"button_f1", '('},		/* paropen or swapxy */
-		{"button_f2", ')'},		/* parclose or rolldn */
+		{"button_pow", "^", '^'},
+		{"button_lsh", "<<", '<'},
+		{"button_mod", "MOD", 'm'},
+		{"button_and", "AND", '&'},
+		{"button_or", "OR", '|'},
+		{"button_xor", "XOR", 'x'},
+		{"button_enter", "=", '='},
+		{"button_plus", "+", '+'},
+		{"button_minus", "-", '-'},
+		{"button_mult", "*", '*'},
+		{"button_div", "/", '/'},
+		{"button_percent", "%", '%'},
+		{"button_f1", "(", '('},	/* paropen or swapxy */
+		{"button_f2", ")", ')'},	/* parclose or rolldn */
 		{NULL}
 	};
 	
 	s_gfunc_map gfunc_map[] = {
-		{"button_sign", display_result_toggle_sign},
-		{"button_ee", display_append_e},
-		{"button_f1", gfunc_f1},	/* paropen or swapxy */
-		{"button_f2", gfunc_f2},	/* parclose or rolldn */
+		{"button_sign", "-", display_result_toggle_sign},
+		{"button_ee", "e", display_append_e},
+		{"button_f1", "(", gfunc_f1},	/* paropen or swapxy */
+		{"button_f2", ")", gfunc_f2},	/* parclose or rolldn */
 		{NULL}
 	};
 	
-	s_function_map function_map[] = {
-		{"button_sin", {sin, asin, sinh, asinh}, TRUE},
-		{"button_cos", {cos, acos, cosh, acosh}, TRUE},
-		{"button_tan", {tan, atan, tanh, atanh}, TRUE},
-		{"button_log", {log10, pow10y, log10, log10}, FALSE},
-		{"button_ln", {log, exp, log, log}, FALSE},
-		{"button_sq", {powx2, sqrt, powx2, powx2}, FALSE},
-		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},
-		{"button_fac", {factorial, factorial, factorial, factorial}, FALSE},
-		{"button_cmp", {cmp, cmp, cmp, cmp}, FALSE},
+	/* declare this one static as we need the display_names throughout */
+	
+	static s_function_map function_map[] = {
+		{"button_sin", {"sin(", "asin(", "sinh(", "asinh("}, {sin_wrapper, asin_wrapper, sinh, asinh}},
+		{"button_cos", {"cos(", "acos(", "cosh(", "acosh("}, {cos_wrapper, acos_wrapper, cosh, acosh}},
+		{"button_tan", {"tan(", "atan(", "tanh(", "atanh("}, {tan_wrapper, atan_wrapper, tanh, atanh}},
+		{"button_log", {"log(", "10^", "log(", "log("}, {log10, pow10y, log10, log10}},
+		{"button_ln", {"ln(", "e^", "ln(", "ln("}, {log, exp, log, log}},
+		{"button_sq", {"^2", "sqrt(", "^2", "^2"}, {powx2, sqrt, powx2, powx2}},
+		{"button_sqrt", {"sqrt(", "^2", "sqrt(", "sqrt("}, {sqrt, powx2, sqrt, sqrt}},
+		{"button_fac", {"!", "!", "!", "!"}, {factorial, factorial, factorial, factorial}},
+		{"button_cmp", {"~", "~", "~", "~"}, {cmp, cmp, cmp, cmp}},
 		{NULL}
 	};
 
@@ -188,26 +196,26 @@ static void set_scientific_object_data ()
 static void set_basic_object_data ()
 {
 	s_operation_map	operation_map[] = {
-		{"button_enter", '='},
-		{"button_plus", '+'},
-		{"button_minus", '-'},
-		{"button_mult", '*'},
-		{"button_div", '/'},
-		{"button_percent", '%'},
-		{"button_f1", '('},		/* paropen or swapxy */
-		{"button_f2", ')'},		/* parclose or rolldn */
+		{"button_enter", "=", '='},
+		{"button_plus", "+", '+'},
+		{"button_minus", "-", '-'},
+		{"button_mult", "*", '*'},
+		{"button_div", "/", '/'},
+		{"button_percent", "%", '%'},
+		{"button_f1", "(", '('},	/* paropen or swapxy */
+		{"button_f2", ")", ')'},	/* parclose or rolldn */
 		{NULL}
 	};
 	
 	s_gfunc_map gfunc_map[] = {
-		{"button_sign", display_result_toggle_sign},
-		{"button_f1", gfunc_f1},	/* paropen or swapxy */
-		{"button_f2", gfunc_f2},	/* parclose or rolldn */
+		{"button_sign", "-", display_result_toggle_sign},
+		{"button_f1", "(", gfunc_f1},	/* paropen or swapxy */
+		{"button_f2", ")", gfunc_f2},	/* parclose or rolldn */
 		{NULL}
 	};
 	
 	s_function_map function_map[] = {
-		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},
+		{"button_sqrt", {"sqrt", "^2", "sqrt", "sqrt"}, {sqrt, powx2, sqrt, sqrt}},
 		{NULL}
 	};
 	
@@ -223,13 +231,16 @@ static void set_disp_ctrl_object_data ()
 	int	counter=0;
 	
 	s_gfunc_map map[] = {\
-		{"button_clr", clear},\
-		{"button_backspace", display_result_backspace},\
-		{"button_allclr", all_clear},\
+		{"button_clr", NULL, clear},\
+		{"button_backspace", NULL, backspace},\
+		{"button_allclr", NULL, all_clear},\
 		{NULL}\
 	};
 
 	while (map[counter].button_name != NULL) {
+		g_object_set_data (G_OBJECT (glade_xml_get_widget (
+			dispctrl_xml, map[counter].button_name)),
+			"display_string", map[counter].display_string);
 		g_object_set_data (G_OBJECT (glade_xml_get_widget (
 			dispctrl_xml, map[counter].button_name)),
 			"func", map[counter].func);
@@ -296,6 +307,7 @@ GtkWidget *ui_main_window_create ()
 void ui_main_window_set_dispctrl (int location)
 {
 	GtkWidget	*table_dispctrl, *box;
+	s_signal_cb	signal_cb;
 	
 	/* destroy any existing display controls */
 	if (dispctrl_xml) {
@@ -330,6 +342,12 @@ void ui_main_window_set_dispctrl (int location)
 			error_message ("Unknown mode in \"ui_main_window_set_dispctrl\"");
 	}
 	set_disp_ctrl_object_data ();
+
+	/* finally we connect that signal handler */
+	
+	signal_cb.detailed_signal = g_strdup ("button_press_event");
+	signal_cb.callback = (GCallback) on_button_press_event;
+	set_all_dispctrl_buttons_property (set_table_child_callback, (gpointer) &signal_cb);
 }
 
 /* ui_main_window_buttons_destroy. removes the scientific resp basic mode 
@@ -354,22 +372,27 @@ void ui_main_window_buttons_destroy ()
 
 void ui_main_window_buttons_create (int mode)
 {
-	GtkWidget	*box;
+	GtkWidget	*box, *button;
 	struct lconv 	*locale_settings;
+	s_signal_cb	signal_cb;
 	
 	if (mode == BASIC_MODE) {
 		button_box_xml = glade_file_open (BASIC_GLADE_FILE, "button_box", TRUE);
 		box = glade_xml_get_widget (main_window_xml, "window_vbox");
-		ui_pack_from_xml (box, 3, button_box_xml, "button_box", 
+		ui_pack_from_xml (box, 4, button_box_xml, "button_box", 
 			"button_1", TRUE, TRUE);
 		set_basic_object_data (button_box_xml);
 	} else if (mode == SCIENTIFIC_MODE) {
 		button_box_xml = glade_file_open (SCIENTIFIC_GLADE_FILE, "button_box", TRUE);
 		box = glade_xml_get_widget (main_window_xml, "window_vbox");
-		ui_pack_from_xml (box, 3, button_box_xml, "button_box", 
+		ui_pack_from_xml (box, 4, button_box_xml, "button_box", 
 			"button_1", TRUE, TRUE);
 		set_scientific_object_data (button_box_xml);
 	} else error_message ("Unknown mode in \"ui_main_window_buttons_create\"");
+
+	signal_cb.detailed_signal = g_strdup ("button_press_event");
+	signal_cb.callback = (GCallback) on_button_press_event;
+	set_all_normal_buttons_property (set_table_child_callback, (gpointer) &signal_cb);
 	
 	/* update "decimal point" button to locale's decimal point */
 	dec_point[0] = DEFAULT_DEC_POINT;
@@ -382,6 +405,26 @@ is not supported: >%s<\nYou might face problems when using %s! %s\n)"),
 	dec_point[1] = '\0';
 	gtk_button_set_label ((GtkButton *) glade_xml_get_widget (
 		button_box_xml, "button_point"), dec_point);
+	/* disable mr and m+ button if there is nothing to display */
+	button = glade_xml_get_widget (button_box_xml, "button_MR");
+	gtk_widget_set_sensitive (button, memory.len > 0);
+	button = glade_xml_get_widget (button_box_xml, "button_Mplus");
+	gtk_widget_set_sensitive (button, memory.len > 0);
+}
+
+/* set_table_child_callback. Function argument for set_all_*_buttons_property.
+ * sets the size.
+ */
+
+static void set_table_child_callback (gpointer data, gpointer user_data)
+{
+	s_signal_cb	*signal_cb;
+	GtkTableChild	*table_child;
+	
+	table_child = data;
+	signal_cb = user_data;
+	g_signal_connect (table_child->widget, signal_cb->detailed_signal, 
+		signal_cb->callback, NULL);
 }
 
 /* set_table_child_size. Function argument for set_all_buttons_property.
@@ -421,20 +464,31 @@ static void set_table_child_font (gpointer data, gpointer user_data)
 	/* else do nothing */
 }
 
-/* set_all_buttons_property. calls func with argument data for every button.
+/* set_all_dispctrl_buttons_property. calls func with argument data for 
+ * every button of the display control section.
  */
 
-static void set_all_buttons_property (GFunc func, gpointer data)
+static void set_all_dispctrl_buttons_property (GFunc func, gpointer data)
 {
 	GtkTable	*table;
-	
-	/* at first the display control table. always there; somehor */
+
+	/* at first the display control table. always there; somehow */
 	table = (GtkTable *) glade_xml_get_widget (dispctrl_xml, 
 		"table_dispctrl");
+	
 	/* dispctrl_right has an extra table for cosmetic reasons. */
 	if (GTK_IS_TABLE (((GtkTableChild *)table->children->data)->widget))
 		table = (GtkTable *) ((GtkTableChild *)table->children->data)->widget;
 	g_list_foreach (table->children, func, data);
+}
+
+/* set_all_dispctrl_buttons_property. calls func with argument data for 
+ * every button of the "calculator" section.
+ */
+
+static void set_all_normal_buttons_property (GFunc func, gpointer data)
+{
+	GtkTable	*table;
 	
 	/* now depending on mode the remaining buttons */
 	if (prefs.mode == BASIC_MODE) {
@@ -456,6 +510,15 @@ static void set_all_buttons_property (GFunc func, gpointer data)
 	else error_message ("Unknown mode in \"set_all_buttons_property\"");
 }
 
+/* set_all_buttons_property. calls func with argument data for every button.
+ */
+
+static void set_all_buttons_property (GFunc func, gpointer data)
+{
+	set_all_dispctrl_buttons_property (func, data);
+	set_all_normal_buttons_property (func, data);
+}
+
 /* set_all_buttons_size. gateway for set_all_buttons_property.
  */
 
@@ -499,7 +562,8 @@ void update_active_buttons (int number_base, int notation_mode)
 	GtkWidget	*current_button;
 	unsigned int	state;
 	
-	state = (1 << number_base) | (1 << (notation_mode + NR_NUMBER_BASES));
+	/* state = (1 << number_base) | (1 << (notation_mode + NR_NUMBER_BASES)); */
+	state = 1 << number_base;
 	while (active_buttons[counter].button_name != NULL) {
 		current_button = glade_xml_get_widget (button_box_xml, 
 			active_buttons[counter].button_name);
@@ -513,6 +577,19 @@ void update_active_buttons (int number_base, int notation_mode)
 	}
 }
 
+void update_dispctrl()
+{
+	/* just put one here and hide it afterwards. we need the button
+			for working key accelerators. */
+	if (prefs.mode == BASIC_MODE) 
+		ui_main_window_set_dispctrl (DISPCTRL_BOTTOM);
+	else if (current_status.notation == CS_RPN)
+		ui_main_window_set_dispctrl (DISPCTRL_RIGHTV);
+	else ui_main_window_set_dispctrl (DISPCTRL_RIGHT);
+	set_widget_visibility (dispctrl_xml, "table_dispctrl", 
+		prefs.vis_dispctrl);
+}
+
 /*
  * (un) hide a widget
  */ 
@@ -544,7 +621,6 @@ GtkWidget *ui_font_dialog_create (char *title, GtkButton *button)
 	return font_dialog;
 }
 
-
 GtkWidget *ui_color_dialog_create (char *title, GtkButton *button)
 {
 	GladeXML	*color_xml;
@@ -632,6 +708,27 @@ void position_menu (GtkMenu *menu,
 	*push_in = TRUE;
 }
 
+GtkWidget *ui_user_functions_menu_create (s_user_function *user_function, GCallback user_function_handler)
+{
+	GtkWidget	*menu, *child;
+	int		counter=0;
+	char		*label;
+	
+	menu = gtk_menu_new();
+	while (user_function[counter].name != NULL) {
+		label = g_strdup_printf ("%s(%s) = %s", user_function[counter].name, 
+			user_function[counter].variable, 
+			user_function[counter].expression);
+		child = gtk_menu_item_new_with_label(label);
+		g_free (label);
+		gtk_menu_shell_append ((GtkMenuShell *) menu, child);
+		gtk_widget_show (child);
+		g_signal_connect (G_OBJECT (child), "activate", user_function_handler, GINT_TO_POINTER(counter));
+		counter++;
+	}
+	return menu;
+}
+
 GtkWidget *ui_constants_menu_create (s_constant *constant, GCallback const_handler)
 {
 	GtkWidget	*menu, *child;
@@ -639,7 +736,7 @@ GtkWidget *ui_constants_menu_create (s_constant *constant, GCallback const_handl
 	char		*label;
 	
 	menu = gtk_menu_new();
-	while (constant[counter].desc != NULL) {
+	while (constant[counter].name != NULL) {
 		label = g_strdup_printf ("%s: %s (%s)", constant[counter].name, constant[counter].value, constant[counter].desc);
 		child = gtk_menu_item_new_with_label(label);
 		g_free (label);
@@ -651,7 +748,7 @@ GtkWidget *ui_constants_menu_create (s_constant *constant, GCallback const_handl
 	return menu;
 }
 
-GtkWidget *ui_memory_menu_create (s_array memory, GCallback const_handler, char *last_item)
+GtkWidget *ui_memory_menu_create (s_array memory, GCallback memory_handler, char *last_item)
 {
 	GtkWidget	*menu, *child;
 	int		counter=0;
@@ -664,7 +761,7 @@ GtkWidget *ui_memory_menu_create (s_array memory, GCallback const_handler, char
 		g_free (label);
 		gtk_menu_shell_append ((GtkMenuShell *) menu, child);
 		gtk_widget_show (child);
-		g_signal_connect (G_OBJECT (child), "activate", const_handler, GINT_TO_POINTER(counter));
+		g_signal_connect (G_OBJECT (child), "activate", memory_handler, GINT_TO_POINTER(counter));
 	}
 	if (last_item != NULL) {
 		label = g_strdup (last_item);
@@ -672,11 +769,26 @@ GtkWidget *ui_memory_menu_create (s_array memory, GCallback const_handler, char
 		g_free (label);
 		gtk_menu_shell_append ((GtkMenuShell *) menu, child);
 		gtk_widget_show (child);
-		g_signal_connect (G_OBJECT (child), "activate", const_handler, GINT_TO_POINTER(counter));
+		g_signal_connect (G_OBJECT (child), "activate", memory_handler, GINT_TO_POINTER(counter));
 	}
 	return menu;
 }
 
+GtkWidget *ui_right_mouse_menu_create ()
+{
+	GtkWidget	*menu, *menu_item;
+	
+	menu = gtk_menu_new();
+	
+	menu_item = gtk_check_menu_item_new_with_mnemonic (_("Show _menu bar"));
+	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menu_item, prefs.show_menu);
+	gtk_menu_shell_append ((GtkMenuShell *) menu, menu_item);
+	gtk_widget_show (menu_item);
+	g_signal_connect (G_OBJECT (menu_item), "activate", (GCallback) on_show_menubar1_activate, NULL);
+	
+	return menu;
+}
+
 GtkWidget *ui_pref_dialog_create ()
 {
 	int			counter=0;
@@ -719,32 +831,69 @@ GtkWidget *ui_pref_dialog_create ()
 
 	/* make user defined constants list. */
 	
-	store = gtk_list_store_new (NR_CONST_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	prefs_constant_store = gtk_list_store_new (NR_CONST_COLUMNS, 
+		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	counter = 0;
-	while (constant[counter].desc != NULL) {
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 
-			NAME_COLUMN, constant[counter].name, 
-			VALUE_COLUMN, constant[counter].value, 
-			DESC_COLUMN, constant[counter].desc,
+	while (constant[counter].name != NULL) {
+		gtk_list_store_append (prefs_constant_store, &iter);
+		gtk_list_store_set (prefs_constant_store, &iter, 
+			CONST_NAME_COLUMN, constant[counter].name, 
+			CONST_VALUE_COLUMN, constant[counter].value, 
+			CONST_DESC_COLUMN, constant[counter].desc,
 			-1);
 		counter++;
 	}
-	tree_view = glade_xml_get_widget (prefs_xml, "treeview1");
-	gtk_tree_view_set_model ((GtkTreeView *) tree_view, GTK_TREE_MODEL (store));
+	tree_view = glade_xml_get_widget (prefs_xml, "constant_treeview");
+	gtk_tree_view_set_model ((GtkTreeView *) tree_view, 
+		GTK_TREE_MODEL (prefs_constant_store));
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", NAME_COLUMN, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, 
+		"text", CONST_NAME_COLUMN, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
-	column = gtk_tree_view_column_new_with_attributes ("Value", renderer, "text", VALUE_COLUMN, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("Value", renderer, 
+		"text", CONST_VALUE_COLUMN, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
-	column = gtk_tree_view_column_new_with_attributes ("Description", renderer, "text", DESC_COLUMN, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("Description", renderer, 
+		"text", CONST_DESC_COLUMN, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
 	g_signal_connect (G_OBJECT (select), "changed",
                   G_CALLBACK (const_list_selection_changed_cb),
                   NULL);
+	
+	/* make user defined function list */
 
+	prefs_user_function_store = gtk_list_store_new (NR_UFUNC_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	counter = 0;
+	while (user_function[counter].name != NULL) {
+		gtk_list_store_append (prefs_user_function_store, &iter);
+		gtk_list_store_set (prefs_user_function_store, &iter, 
+			UFUNC_NAME_COLUMN, user_function[counter].name, 
+			UFUNC_VARIABLE_COLUMN, user_function[counter].variable, 
+			UFUNC_EXPRESSION_COLUMN, user_function[counter].expression,
+			-1);
+		counter++;
+	}
+	tree_view = glade_xml_get_widget (prefs_xml, "user_function_treeview");
+	gtk_tree_view_set_model ((GtkTreeView *) tree_view, 
+		GTK_TREE_MODEL (prefs_user_function_store));
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, 
+		"text", UFUNC_NAME_COLUMN, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
+	column = gtk_tree_view_column_new_with_attributes ("Variable", renderer, 
+		"text", UFUNC_VARIABLE_COLUMN, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
+	column = gtk_tree_view_column_new_with_attributes ("Expression", renderer, 
+		"text", UFUNC_EXPRESSION_COLUMN, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
+	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+	g_signal_connect (G_OBJECT (select), "changed",
+                  G_CALLBACK (user_function_list_selection_changed_cb),
+                  NULL);	
+	
 	/* pack widget of same size into GtkSizeGroup */
 
 	sgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -765,6 +914,16 @@ GtkWidget *ui_pref_dialog_create ()
 	gtk_size_group_add_widget (sgroup,
 		glade_xml_get_widget (prefs_xml, "prefs_const_clear_button"));
 	
+	sgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	gtk_size_group_add_widget (sgroup, 
+		glade_xml_get_widget (prefs_xml, "prefs_func_add_button"));
+	gtk_size_group_add_widget (sgroup,
+		glade_xml_get_widget (prefs_xml, "prefs_func_update_button"));
+	gtk_size_group_add_widget (sgroup,
+		glade_xml_get_widget (prefs_xml, "prefs_func_delete_button"));
+	gtk_size_group_add_widget (sgroup,
+		glade_xml_get_widget (prefs_xml, "prefs_func_clear_button"));
+	
 	sgroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget (sgroup, 
 		glade_xml_get_widget (prefs_xml, "prefs_hex_bits_label"));
@@ -786,3 +945,98 @@ GtkWidget *ui_about_dialog_create()
 	glade_xml_signal_autoconnect(about_dialog_xml);
 	return glade_xml_get_widget (about_dialog_xml, "about_dialog");
 }
+
+void ui_formula_entry_activate ()
+{
+	GtkWidget	*formula_entry;
+	
+	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	gtk_widget_activate(formula_entry);
+}
+
+void ui_formula_entry_set (G_CONST_RETURN gchar *text)
+{
+	GtkWidget	*formula_entry;
+
+	if (text == NULL) return;
+	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	gtk_entry_set_text ((GtkEntry *) formula_entry, text);
+}
+
+void ui_formula_entry_insert (G_CONST_RETURN gchar *text)
+{
+	GtkWidget	*formula_entry;
+	int		position;
+	
+	if (text == NULL) return;
+	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	position = gtk_editable_get_position ((GtkEditable *) formula_entry);
+	gtk_editable_insert_text ((GtkEditable *) formula_entry, text, -1,
+                                             &position);
+	gtk_editable_set_position ((GtkEditable *) formula_entry, position);
+}
+
+void ui_formula_entry_backspace ()
+{
+	GtkWidget	*formula_entry;
+	
+	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	gtk_editable_delete_text ((GtkEditable *) formula_entry, 
+		strlen(gtk_entry_get_text((GtkEntry *) formula_entry)) - 1, -1);
+}
+
+/* ui_formula_entry_state. if color == NULL looks like we get default. this is
+ * what we want.
+ */
+
+void ui_formula_entry_state (gboolean error)
+{
+	GtkWidget		*formula_entry;
+	GdkColor		*color=NULL;
+	
+	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	if (error) {
+		color = (GdkColor *) malloc(sizeof(GdkColor));
+		gdk_color_parse ("red", color);
+	}
+	gtk_widget_modify_text (formula_entry, 0, color);
+	gtk_widget_modify_text (formula_entry, 1, color);
+	gtk_widget_modify_text (formula_entry, 2, color);
+	gtk_widget_modify_text (formula_entry, 3, color);
+	gtk_widget_modify_text (formula_entry, 4, color);
+	if (color) g_free(color);
+}
+
+void ui_button_set_pan ()
+{
+	set_button_label_and_tooltip (button_box_xml, "button_enter", 
+		_("="), _("Enter"));
+	set_button_label_and_tooltip (button_box_xml, "button_pow", 
+		_("x^y"), _("Power"));
+	set_button_label_and_tooltip (button_box_xml, "button_f1", 
+		_("("), _("Open Bracket"));
+	set_button_label_and_tooltip (button_box_xml, "button_f2", 
+		_(")"), _("Close Bracket"));
+}
+
+void ui_button_set_rpn ()
+{
+	set_button_label_and_tooltip (button_box_xml, "button_enter", 
+		_("ENT"), _("Enter"));
+	set_button_label_and_tooltip (button_box_xml, "button_pow", 
+		_("y^x"), _("Power"));
+	set_button_label_and_tooltip (button_box_xml, "button_f1", 
+		_("x<>y"), _("swap current number with top of stack"));
+	set_button_label_and_tooltip (button_box_xml, "button_f2", 
+		_("roll"), _("roll down stack"));	
+}
+
+void ui_relax_fmod_buttons ()
+{
+	GtkWidget	*tbutton;
+	
+	tbutton = glade_xml_get_widget (button_box_xml, "button_inv");
+	gtk_toggle_button_set_active ((GtkToggleButton *) tbutton, FALSE);
+	tbutton = glade_xml_get_widget (button_box_xml, "button_hyp");
+	gtk_toggle_button_set_active ((GtkToggleButton *) tbutton, FALSE);
+}
